﻿/*
  USART (multi)

  半二重のRS-485ないしTTL I/Fによるシリアル通信に加え、CN1のGPIOを介して
  でシリアル通信ができるように構成。

  ・GPIO0(CN1-7)を送信、GPIO1(CN1-6)を受信としたので、それらの端子に何か
    しらの装置をつなげて相互通信する
  ・1秒周期でUSART0とUSART1から1文字送信し続ける
  ・USART0で受信したーデータはUSART1から送信、USART1で受信したデータは
    USART0から送信する
  ・外付けの装置を設けられな, い場合はGPIO0とGPIO1を短絡させ
     USART0 RX -> USART1 TX -> ジャンパ -> USART1 RX -> USART0 TX
    という遠回しの経路によるエコーバックを作って動作確認ができる
*/

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
// USART用関数
#include  <us.h>
#include  <wl_delay.h>

// マクロ・グローバル変数など
//-----------------------------
// 端子機能追加分
static const TPin pins[] = {
  { _GPIO0, PIO_TYPE_MOVABLE, SWM_U1_TXD_O, PIO_MODE_DEFAULT },  // USART1 TXD
  { _GPIO1, PIO_TYPE_MOVABLE, SWM_U1_RXD_I, PIO_MODE_PULLUP  },  // USART1 RXD
};

// main
//-----------------------------
void main (void) {
  // シリアル通信バッファ
  uint8_t txd0[500], rxd0[500];
  uint8_t txd1[500], rxd1[500];

  // 追加の端子機能の初期化
  PIO_Configure (pins, PIO_LISTSIZE (pins));

  // USART0を初期化
  usart0_dma_init (Chip_Clock_GetFRGClockRate (0), 115200, USIP_8N1, txd0, sizeof (txd0), rxd0, sizeof (rxd0));
  // USART1のクロックソースとしてFRG0を選択(初期化ルーチンでは触れていないので)
  LPC_SYSCON->UART1CLKSEL = SYSCON_FLEXCOMMCLKSELSRC_FRG0;
  // USART1を初期化
  usart1_dma_init (Chip_Clock_GetFRGClockRate (0), 9600, USIP_8N1, txd1, sizeof (txd1), rxd1, sizeof (rxd1));

  usart0_dma_puts ("\nHELLO (from USART0)!\n");
  usart1_dma_puts ("\nHELLO (from USART1)!\n");

  uint32_t t = Delay_GetTick();
  char r0 = 0, r1 = 0;
  for (;;) {
    // USART0 RX -> USART1 TX
    while (usart0_dma_rx_buff()) usart1_dma_putc (r0 = usart0_dma_getc());
    // USART1 RX -> USART0 TX
    while (usart1_dma_rx_buff()) usart0_dma_putc (r1 = usart1_dma_getc());
    if (r0 == '!') {
      usart0_dma_puts ("\nBYE!\n ");
      Delay_ms (50);
      NVIC_SystemReset();
    }
    // 1秒周期で1文字送信
    if (Delay_GetTick() - t > 1000) {
      usart0_dma_putc ('*');
      usart1_dma_putc ('+');
      t = Delay_GetTick();
    }

    // コンフリクト回避(ごまかしのウェイト)
    Delay_ms (50);
  }
}
