﻿/*
  I2C

  コードの大半はBNO055の設定絡みで、I2Cらしいところは imu_write_byte・
  imu_read_byte・imu_read_datavalue関数程度。

  ・BNO055用のコードは多少汎用性があるので e097b_imu.h に宣言
  ・LEDはGPIOではなくSCTを使用してPWMを作り輝度を変化させている
 */

#include  <us.h>
#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
// IMU用サブルーチン
#include  "e097b_imu.h"
#include  <us.h>
// I2C用関数
#include  <wl_i2c.h>
#include  <wl_delay.h>

// マクロ・グローバル変数など
//-----------------------------
#define cls()           usart0_dma_puts ("\x1b[2J");
#define gotoxy(xx, yy)  usart0_dma_printf ("\x1b[%d;%dH", yy + 1, xx + 1);

// LED (調子こいてSCTでPWM)
//-----------------------------
#define SCT_PWM_PIN_LED    0    // COUT0
#define SCT_PWM_LED        1    // Index of LED PWM (1..)

void LED_Init (void) {
  const TPin p = { _LED2, PIO_TYPE_MOVABLE, SWM_SCTOUT_0_O, PIO_MODE_DEFAULT }; // PIO0_24    SCT0

  PIO_Configure (&p, 1);

  Chip_SCTPWM_Init (LPC_SCT);
  Chip_SCTPWM_SetRate (LPC_SCT, 10000); // 10kHz
  Chip_SCTPWM_SetOutPin (LPC_SCT, SCT_PWM_LED, SCT_PWM_PIN_LED);
  Chip_SCTPWM_SetDutyCycle (LPC_SCT, SCT_PWM_LED, Chip_SCTPWM_GetTicksPerCycle (LPC_SCT) / 2);
  Chip_SCTPWM_Start (LPC_SCT);

  Chip_SCTPWM_SetDutyCycle (LPC_SCT, SCT_PWM_LED, Chip_SCTPWM_GetTicksPerCycle (LPC_SCT));  // 消灯
}

void LED_SetBright (int permil) {
  if (permil > 1000) permil = 1000;
  Chip_SCT_SetMatchReload (LPC_SCT, (CHIP_SCT_MATCH_REG_T)SCT_PWM_LED, Chip_SCTPWM_GetTicksPerCycle (LPC_SCT) - (Chip_SCTPWM_GetTicksPerCycle (LPC_SCT) * permil) / 1000);
}

bool led_on = false;

void LED_Toggle (void) {
  bool On = !led_on;
  LED_SetBright (On ? 30 : 0);
  led_on = On;
}

void LED_Set (bool On) {
  LED_SetBright (On ? 30 : 0);
  led_on = On;
}

// main
//-----------------------------
void main (void) {

  const char *modest[] = {"CONFIG", "ACCONLY", "MAGONLY", "GYRONLY", "ACCMAG", "ACCGYRO", "MAGGYRO", "AMG", "IMUPLUS", "COMPASS", "M4G", "NDOF_FMC_OFF", "NDOF"};

  void change_opmode (TOperationMode m) {
    imu_set_opmode (m, NULL);
    usart0_dma_puts (">OpMode=");
    usart0_dma_puts (modest[m]);
  }

  uint8_t txb[100], rxb[10];

  // USART初期化
  usart0_dma_init (Chip_Clock_GetFRGClockRate (0), 115200, USIP_8N1, txb, sizeof (txb), rxb, sizeof (rxb));
  // LED初期化
  LED_Init();
  // I2C初期化
  // クロック源の選択はライブラリで処置していないのでここで
  LPC_SYSCON->I2C0CLKSEL = SYSCON_FLEXCOMMCLKSELSRC_MAINCLK;
  IIC_Init (LPC_I2C0, IIC_Fm);
  LPC_I2C0->CFG |= I2C_CFG_MONCLKSTR;

  cls();
  TOperationMode mode = opmode_NDOF;

  // IMU初期化
  if (init_imu (mode, axisplace_P0)) {
    Tbno055 d;
    uint8_t tst = 0, stat, err;
    while (1) {
      if (imu_read_datavalue (&d)) {
        gotoxy (0, 1);
        usart0_dma_printf ("OpMode=%s\n", modest[mode]);
        usart0_dma_printf ("acc:%6d,%6d,%6d\n", d.acc_x, d.acc_y, d.acc_z);
        usart0_dma_printf ("mag:%6d,%6d,%6d\n", d.mag_x, d.mag_y, d.mag_z);
        usart0_dma_printf ("gyr:%6d,%6d,%6d\n", d.gyro_x, d.gyro_y, d.gyro_z);
        usart0_dma_printf ("eul:%6d,%6d,%6d\n", d.heading, d.roll, d.pitch);
        usart0_dma_printf ("qua:%6d,%6d,%6d,%6d\n", d.q_w, d.q_x, d.q_y, d.q_z);
        usart0_dma_printf ("lia:%6d,%6d,%6d\n", d.lia_x, d.lia_y, d.lia_z);
        usart0_dma_printf ("grv:%6d,%6d,%6d\n", d.grv_x, d.grv_y, d.grv_z);
        usart0_dma_printf ("tmp;   %3d\n", d.temp);
        if (imu_get_tst_stat_err (NULL, &stat, &err)) usart0_dma_printf ("sys: $%02x, %d, %d  ", tst, stat, err);
      }
      if (usart0_dma_rx_buff()) {
        cls();
        switch (usart0_dma_getc()) {
          // 「0」～「9」「a」～「c」でOperation modeを切り替え
          case '0':
            change_opmode (mode = opmode_CONFIG);
            break;
          case '1':
            change_opmode (mode = opmode_ACCONLY);
            break;
          case '2':
            change_opmode (mode = opmode_MAGONLY);
            break;
          case '3':
            change_opmode (mode = opmode_GYRONLY);
            break;
          case '4':
            change_opmode (mode = opmode_ACCMAG);
            break;
          case '5':
            change_opmode (mode = opmode_ACCGYRO);
            break;
          case '6':
            change_opmode (mode = opmode_MAGGYRO);
            break;
          case '7':
            change_opmode (mode = opmode_AMG);
            break;
          case '8':
            change_opmode (mode = opmode_IMUPLUS);
            break;
          case '9':
            change_opmode (mode = opmode_COMPASS);
            break;
          case 'a':
            change_opmode (mode = opmode_M4G);
            break;
          case 'b':
            change_opmode (mode = opmode_NDOF_FMC_OFF);
            break;
          case 'c':
            change_opmode (mode = opmode_NDOF);
            break;

          // 「s」「l」「n」でPower modeを切り替え
          case 's':
            usart0_dma_puts (">PowMode=SUSPEND");
            imu_set_powermode (powmode_SUSPEND);
            break;
          case 'l':
            usart0_dma_puts (">PowMode=LOWPOWER");
            imu_set_powermode (powmode_LOWPOWER);
            break;
          case 'n':
            usart0_dma_puts (">PowMode=NORMAL");
            imu_set_powermode (powmode_NORMAL);
            break;

          // 「t」でセルフテスト
          case 't':
            usart0_dma_puts (">SelfTest");
            imu_get_tst_stat_err (&tst, &stat, &err);
            break;

          // 「i」で再初期化
          case 'i':
            usart0_dma_puts (">Init");
            init_imu (mode, axisplace_P0);
            break;

          // 「!」でプログラム終了
          case '!':
            imu_set_powermode (powmode_SUSPEND);
            NVIC_SystemReset();
            break;
        }
      }
      LED_Toggle();
      Delay_ms (100);
    }
  }
}
